/*
    Copyright (C) 2012  Leonardo Pozzati
    This file is part of MetastableRandWalk.

    MetastableRandWlak is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    MetastableRandWalk is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Nome-Programma.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "libDens.h"

using namespace std;

							//LIBDENS//
//FRAME DENS
unsigned long frRecog(unsigned long step,
					  signed long x,
					  signed long y) {
	unsigned long a = abs(x) > abs(y) ? abs(x) : abs(y);
	if(a == 0) {
		return 1;
	} else if(a % step != 0) {
		return (a / step) + 1;
	} else {
		return a / step;
	}
}

unsigned long squareSites(unsigned long side) {
	return (2 * side + 1)*(2 * side + 1);
}

map<unsigned long, pair <unsigned long, unsigned long > >
frCountVsNormInit(unsigned long bound, unsigned long step) {
	map<unsigned long,pair<unsigned long,unsigned long> > frCountVsNorm;
	unsigned long external = 0;
	unsigned long internal = 0;
	unsigned long counter;
	for(counter=0; counter<(bound/step); counter++) {
		external = squareSites( (counter+1)*step );
		frCountVsNorm.insert(make_pair(counter+1,
							 make_pair(0, external - internal)));
		internal = external;
	}
	if(bound % step != 0) {
		frCountVsNorm.insert(make_pair(counter+1,
							 make_pair(0,
									  (2*bound+1)*(2*bound+1)-external))
							);
	}
	(frCountVsNorm[1]).second -= 1;
	return frCountVsNorm;
}

void frCountVsNormUpdate(unsigned long step,
						 list< pair <signed long, signed long> > state,
						 map<unsigned long,
							 pair<unsigned long,unsigned long> >
						 &frCountVsNorm) {
	list< pair <signed long, signed long> >::iterator it1;
	unsigned long frame;
	for(it1=state.begin(); it1!=state.end(); it1++) {
		frame = frRecog(step, (*it1).first, (*it1).second);
		frCountVsNorm[frame].first += 1;
	}
}

void frCountVsNormReset(map<unsigned long,
						    pair<unsigned long,unsigned long> >
						&frCountVsNorm) {
	map<unsigned long, pair<unsigned long, unsigned long> >::iterator it1;
	for(it1=frCountVsNorm.begin();it1!=frCountVsNorm.end(); it1++) {
		((*it1).second).first = 0;
	}
}

//CROWN DENS
unsigned long crRecog(unsigned long bound,
					  unsigned long step,
					  signed long x,
					  signed long y) {
	double a = sqrt((x*x) + (y*y));
	if(a == 0) {
		return 1;
	} else if(a > bound) {
		return bound / step + 1;
	} else if(fmod(a, step) != 0) {
		//per int() si intende il massimo numero intero minore di () 
		return int(a / step) + 1;
	} else {
		//per int() si intende il massimo numero intero minore di ()
		return int(a / step);
	}
}

int externalSitesCheck(signed long x,
					   signed long y,
					   unsigned long r) {
		if((x*x) + (y*y) > (r*r)) {
			return 1;
		}
		else
			return 0;
}

unsigned long circleSites(unsigned long r){
	unsigned long externalSites = 0;
	unsigned long diagonalSites = (unsigned long)(sqrt( (r*r) / 2 ));
	//eseguo la conta dei punti del primo quadrante
	//esclusi quelli sugli assi
	for(unsigned long i = diagonalSites; i > 0; i--) {
		for(unsigned long j = r; j > diagonalSites; j--){
			if(externalSitesCheck(j, i, r)) {
				externalSites += 1;
			}
			else break;
		}
	}
	return 8*((diagonalSites)*(diagonalSites + 1) / 2 +
			  (r - diagonalSites - 1)*(diagonalSites) -
			  externalSites) +
			  4*diagonalSites + 4*r + 1;
}

map<unsigned long, pair <unsigned long, unsigned long > >
crCountVsNormInit(unsigned long bound, unsigned long step) {
	map<unsigned long,pair<unsigned long,unsigned long> > crCountVsNorm;
	unsigned long external = 0;
	unsigned long internal = 0;
	unsigned long counter;
	for(counter=0; counter<(bound/step); counter++) {
		external = circleSites( (counter+1)*step );
		crCountVsNorm.insert(make_pair(counter+1,
							 make_pair(0, external - internal)));
		internal = external;
	}
	crCountVsNorm.insert(make_pair(counter+1,
						 make_pair(0,
								  (2*bound+1)*(2*bound+1)-external))
						);
	(crCountVsNorm[1]).second -= 1;
	return crCountVsNorm;
}

void crCountVsNormUpdate(unsigned long bound,
						 unsigned long step,
						 list< pair <signed long, signed long> > state,
						 map<unsigned long,
							 pair<unsigned long,unsigned long> >
						 &crCountVsNorm) {
	list< pair <signed long, signed long> >::iterator it1;
	unsigned long crown;
	for(it1=state.begin(); it1!=state.end(); it1++) {
		crown =crRecog(bound, step, (*it1).first, (*it1).second);
		crCountVsNorm[crown].first += 1;
	}
}

void crCountVsNormReset(map<unsigned long,
						    pair<unsigned long,unsigned long> >
						&crCountVsNorm) {
	map<unsigned long, pair<unsigned long, unsigned long> >::iterator it1;
	for(it1=crCountVsNorm.begin();it1!=crCountVsNorm.end(); it1++) {
		((*it1).second).first = 0;
	}
}
